function [Matbeta, vec_err_n, vec_nop, Bvec, ITC] = cmp( y, X, flag_warm_start, flag_ls,  mub, prec, Ng)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
% y                 = response vector
% X                 = full dictionary
% flag_warm_start   = if 1, applies warm start; if 0 does not apply
% flag_ls           = LS estimation is performed
% mub               = max. no. of iterations
% prec              = precision
% Ng                = no. of points on the grid for Bbar
%Output:
% Matbeta           = p X Ng (estimated coefficients)
% vec_err_n =       = Ng X 1 (RSS/n)
% vec_nop           = Ng X 1 (no. of linear parameters)
% Bvec              = grid for Bbar
% ITC               = has Ng cols
%                     Significance of the rows:
%                     AICC;KIC;KICC;BIC;EBIC;EBIC2;SC;ESC;gMDL;EgMDL;MMLU;MMLG
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[n,p] = size(X);

%Transform data
y = y-mean(y);
for jj=1:p
    X(:,jj) = X(:,jj) - mean ( X(:,jj) );
    X(:,jj) = sqrt(n)*( X(:,jj)/norm(X(:,jj)) );
end
noy_n = norm(y)^2/n;

c = (X'*y)/n;
D = (X'*X)/n;

%Grid for Bbar
Bvec = Bgrid(y,X,D,c,Ng);

%Initialization
Matbeta     = zeros(p,Ng);
vec_err_n   = Inf*ones(Ng,1);  
vec_nop     = zeros(Ng,1);
ITC         = Inf*ones(12,Ng);
s           = 0;
if flag_ls~=0
    beta_ls = zeros(p,1);
end



%Loop B
for ind=1:length(Bvec)
    
    %tic    
    
    if s>n-3
        return;
    end

    Bbar = Bvec(ind);
    
    %If NOT warm start
    if (flag_warm_start==0) || (ind==1) 
        start_point = 1;
        beta = zeros(p,1);
        beta_old = beta;
        s = 0;
        delta = beta;
    else
        start_point = 2;
    end

    %Loop iterations
    for j=start_point:mub
    % tic
        if (s> n-3)
            break;
        end
    
        w = 1 - (1/j);
        barw = 1-w;

        a = c -w*delta;

        [maxaj,sj] = max(abs(a));
 
        temp = sign(a(sj))*min(j*maxaj,Bbar);
         
 
        beta = w*beta;
        beta(sj) = beta(sj) + barw*temp;
        delta = w*delta + (barw*temp)*D(:,sj);
        
        
        ff = (beta~=0);
        s = sum(ff);
        
        if max( abs(beta-beta_old) )<prec
            beta_old = beta;
            break;
        else
            beta_old = beta;
        end

    end %for iterations
    
    
    if flag_ls==0
        Matbeta(:,ind) = beta;                             %save  
        est_n = beta'*D*beta;
        err_n = noy_n -2*c'*beta + est_n;
        vec_err_n(ind) = err_n;                            %save 
    else
        %LS estimation
        if sum(beta_ls~=0)<s
            beta_ls = zeros(p,1);
            % beta_ls(ff) = X(:,ff)\y;
            beta_ls(ff) = D(ff,ff)\c(ff);
        end
        Matbeta(:,ind) = beta_ls;                           %save
        est_n = beta_ls'*D*beta_ls;
        err_n = noy_n -2*c'*beta_ls + est_n; 
        vec_err_n(ind) = err_n;                             %save 
    end
    vec_nop(ind) = s;                                       %save
 
    if flag_ls==0
        ITC(:,ind) = comp_itc(n, noy_n, est_n,beta, D(ff,ff), D, err_n, s, p);
    else
        ITC(:,ind) = comp_itc(n, noy_n, est_n,beta_ls, D(ff,ff), D, err_n, s, p);
    end
end %loop B

end %function



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function Bvec = Bgrid(y,X,D,c,Ng)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
% y                 = response vector
% X                 = full dictionary
% D                 = (X'*y)/n
% c                 = (X'*y)/n
% Ng                = no. of points on the grid for Bbar
%Output:
% Bvec              = grid for Bbar
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[n,p] = size(X);

if p>=n 
    lambda_max = max(abs(c));

    lambda_min = lambda_max/n;
    [beta_lasso,~] = lasso(X,y,'Lambda',lambda_min);
    Bmax = sum(abs(beta_lasso));
else

    beta_ls = D\c;
    Bmax = sum(abs(beta_ls));
end %if

CC = log(Ng)/(Ng-1);
Bvec = Bmax*exp( (-CC)*(Ng - (1:Ng)) );

end %function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function vec_itc = comp_itc(n, noy_n, est_n,beta, Dsel, D, err_n, k, p)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
% n             = sample size
% noy_n         = ||y||^2/n
% est_n         = ||\hat{y}||^2/n
% err_n         = RSS/n
% k             = no. of linear param.
% p             = total no. of predictors
%Output:
% vec_itc       = values of the following ITC:
%                 SC; ESC; gMDL; EgMDL; BIC; EBIC1; EBIC2; AICC
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if k<=0
    vec_itc = Inf*ones(8,1);
    return;
end

comb = gammaln(p+1) - gammaln(k+1) - gammaln(p-k+1);

%SC
term_3 = log(k) -(n-1)*log(n-k);
SC = n*log(err_n) +k*log( (est_n/k)/(err_n/(n-k)) ) +term_3;

%ESC
ESC = SC +2*comb;

%gMDL
gMDL = (n/2)*log(noy_n) + (1/2)*log(n);
if (est_n/noy_n) >= (k/n),
    gMDL = (n/2)*log(err_n*n/(n-k)) +(k/2)*log( (est_n/k)/(err_n/(n-k)) ) +log(n);
end

%EgMDL
EgMDL = gMDL +comb;

%BIC
BIC = n*log(err_n) + k*log(n);

%EBIC1
EBIC1 = BIC +2*comb;

%EBIC2
gam = 1; %gam should take values in [0,1]
EBIC2 = BIC + k*4*gam*log(p);

%AICC
AICC = log(err_n) +( 1+k/n )/( 1-(k+2)/n );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%  NEW CRITERIA %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
KIC = n*log(err_n) + 3*(k+1);

KICC = n*log(err_n) + (2*(k+1)*n)/(n-k-2) - n*psi( (n-k)/2 );

te1 = ((n-k)/2)*log(2*pi);
te2 = ((n-k)/2)*( 1+ log(n*err_n/(n-k)) );
te3 = (k/2)*log(pi*n*noy_n);
te4 = -gammaln(1+(k/2));
te5 = (1/2)*log(k+1);
MMLU = te1 + te2 + te3 + te4 + te5;



MMLG = (n/2)*( 1+log((n*err_n)/(n-k+2)) ) + (1/2)*log(n-1) + (1/2); 

md = max(k-2,1);
if ( (k>0) &&  ( est_n/md > (err_n/(n-k+2)) ) )
    MMLG = ((n-k+2)/2)*( 1+log((n*err_n)/(n-k+2)) ) + ((k-2)/2)*log(n*est_n/md) + (1/2)*log((n-k)*k^2);
end



vec_itc = [AICC;KIC;KICC;BIC;EBIC1;EBIC2;SC;ESC;gMDL;EgMDL;MMLU;MMLG];

end %function

    

