function [Matbeta, vec_err, ITC, vec_combr] = omp(y, X)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Based on the implementation from Matlab Central done by
% Author: Sujit Kumar Sahoo, School of Electrical and Electronic
% Engineering, Nanyang Technological University, Singapore.
%Input:
% y         = response vector
% X         = full dictionary

%Output:
% Matbeta   = p X mub (estimated coefficients)
% vec_err   = mub X 1 (RSS)
% ITC       = has mub cols
% Significance of the rows:
% AICC; KIC; KICC; BIC; EBIC1; EBIC2; SC; ESC; gMDL; EgMDL; MMLU; MMLG
% vec_combr  = mub X 1 (log of binomial coefficie nt)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[n,p] = size(X);
maxNumCoef = min(n,p);

Matbeta = zeros(p,maxNumCoef);
vec_err = Inf*ones(maxNumCoef,1);
vec_combr = zeros(maxNumCoef,1);
ITC = Inf*ones(12,maxNumCoef);
a1 = zeros(1,maxNumCoef);

%Transform data
y = y-mean(y);
y1 = y/sqrt(n);
for jj=1:p
    X(:,jj) = X(:,jj) - mean ( X(:,jj) );
    X(:,jj) = sqrt(n)*( X(:,jj)/norm(X(:,jj)) );
    X1(:,jj) = X(:,jj)/norm(X(:,jj));
end

c = (X'*y)/n;
D = (X'*X)/n;

noy_n = norm(y)^2/n;
beta  = zeros(p,1);
delta = zeros(p,1);
k     = 0;
index = zeros(1,maxNumCoef);
A     = [];

currResNorm2 = mean(abs(y1).^2);
while currResNorm2>10^(-6) && k < maxNumCoef
    k = k + 1;
    a = c - delta;

    
     [~,sj]= max(abs(a));
     sj=sj(1);
     index(k)=sj;
        
    
    v = D(index(1:k-1),index(k));
    h = A*v;
    eta = 1/(1-v'*h);
    A = [A+eta*(h*h'), -eta*h
            -eta*h', eta];
    a1(k) = a(index(k))*eta;
    a1(1:k-1) = a1(1:k-1) - a1(k)*h.';
    beta(index(1:k))=a1(1:k);
    Matbeta(index(1:k),k)=a1(1:k);
    delta = D*beta;
    
    residual=y1-X1(:,index(1:k))*a1(1:k).';
    currResNorm2 = mean(abs(residual).^2);
    
    est_n = beta'*delta;
    err_n = noy_n - est_n;
    
    vec_err(k) = err_n; %save
    

    
        
    
    if k==1
        combr = log(p);
    else
        combr = combr + log( (p-k+1)/k );
    end
    vec_combr(k) = combr; %save
        
    ITC(:,k) = comp_itc(n, noy_n, est_n, err_n, beta, D(index(1:k),index(1:k)), c(index(1:k)),combr, k, p);

end



end %function



function vec_itc = comp_itc(n, noy_n, est_n, err_n, bet, Dsel, csel,comb, k, p )

%Input:
% n             sample size
% err_n         RSS/n
% bet           est. linear param.
% Dsel          D(sel,sel)
% csel          c(sel)
% k             no. of linear param.



%SC
term_3 = log(k) -(n-1)*log(n-k);
SC = n*log(err_n) +k*log( (est_n/k)/(err_n/(n-k)) ) +term_3;

%ESC
ESC = SC +2*comb;

%gMDL
gMDL = (n/2)*log(noy_n) + (1/2)*log(n);
if (est_n/noy_n) >= (k/n),
    gMDL = (n/2)*log(err_n*n/(n-k)) +(k/2)*log( (est_n/k)/(err_n/(n-k)) ) +log(n);
end

%EgMDL
EgMDL = gMDL +comb;

%BIC
BIC = n*log(err_n) + k*log(n);

%EBIC1
EBIC1 = BIC +2*comb;

%EBIC2
gam = 1; %gam should take values in [0,1]
EBIC2 = BIC + k*4*gam*log(p);

%AICC
AICC = log(err_n) +( 1+k/n )/( 1-(k+2)/n );


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%  NEW CRITERIA %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
KIC = n*log(err_n) + 3*(k+1);

KICC = n*log(err_n) + (2*(k+1)*n)/(n-k-2) - n*psi( (n-k)/2 );

te1 = ((n-k)/2)*log(2*pi);
te2 = ((n-k)/2)*( 1+ log(n*err_n/(n-k)) );
te3 = (k/2)*log(pi*n*noy_n);
te4 = -gammaln(1+(k/2));
te5 = (1/2)*log(k+1);
MMLU = te1 + te2 + te3 + te4 + te5;



MMLG = (n/2)*( 1+log((n*err_n)/(n-k+2)) ) + (1/2)*log(n-1) + (1/2); 

md = max(k-2,1);
if ( (k>0) &&  ( est_n/md > (err_n/(n-k+2)) ) )
    MMLG = ((n-k+2)/2)*( 1+log((n*err_n)/(n-k+2)) ) + ((k-2)/2)*log(n*est_n/md) + (1/2)*log((n-k)*k^2);
end


vec_itc = [AICC; KIC; KICC; BIC; EBIC1; EBIC2; SC; ESC; gMDL; EgMDL; MMLU; MMLG ];

end %function



