function [Matbeta, vec_err_n, vec_df, ITC, vec_combr,vec_combd] = rmp( y, X, mub)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
% y         = response vector
% X         = full dictionary
% mub       = max. no. of iterations


%Output:
% Matbeta   = p X mub (estimated coefficients)
% vec_err_n = mub X 1 (RSS/n)
% vec_df    = mub X 1 (df)
% ITC       = has mub cols
% Significance of the rows:

%  AICC;KIC;KICC; BIC; EBIC1;EBIC1d; EBIC2;SC1; SC2; ESC1; ESC2; ESC1d;ESC2d;
%  gMDL1; gMDL2; EgMDL1; EgMDL2; EgMDL1d;EgMDL2d;MMLU;MMLG1;MMLG2
% vec_combr  = mub X 1 (log of binomial coefficient)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[n,p] = size(X);
Matbeta = zeros(p,mub);
vec_err_n = Inf*ones(mub,1);
vec_df = Inf*ones(mub,1);
vec_combr = zeros(mub,1);
vec_combd = zeros(mub,1);
ITC = Inf*ones(22,mub);


y = y-mean(y);
for jj=1:p
    X(:,jj) = X(:,jj) - mean ( X(:,jj) );
    X(:,jj) = sqrt(n)*( X(:,jj)/norm(X(:,jj)) );
end

c = (X'*y)/n;
D = (X'*X)/n;

noy = norm(y)^2/n;
beta = zeros(p,1);
delta = zeros(p,1);
k=0;
S = zeros(p,1);
G = eye(n);
df = 0;
sel = [];

for j=1:mub
    if (k>p-1),
        return;
    end
    if df>n-3,
        return;
    end
    
    w = 1-(1/j);
    a = c-w*delta;
    [~,sj] = max(abs(a));
    beta = w*beta;
    beta(sj) = beta(sj) + a(sj);
    delta = w*delta + a(sj)*D(:,sj);
    
    est_n = beta'*delta;
    err_n = noy + est_n - 2*beta'*c;
    
    
    if err_n<=0
        return;
    end

    
    Matbeta(:,j) = beta; %save
    vec_err_n(j) = err_n; %save
    
    %Compute df
    G = eye(n) + G - (1/n)*X(:,sj)*X(:,sj)'*G;
    df = n + n/j - 1/j*trace(G);
    vec_df(j) = df; 
    
    
    if S(sj)==0
        S(sj) = 1;
        k = sum(S);
        if k == 1
            combr = log(p);
        else
            combr = combr + log( (p-k+1)/k );
        end
    end         
     vec_combr(j) = combr;  
    
    %Extended criteria (apply gammaln)
    combd = gammaln(p+1) -gammaln(df+1) -gammaln(p-df+1);
    vec_combd(j) = combd;
    
    
    sel = [sel, sj];
    indx = unique(sel);
    
    vec_itc = comp_itc(j,combd, n, noy, est_n, err_n, df, combr,D(indx,indx),D, beta, p);
    ITC(:,j) = vec_itc; %save
end

end %function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function vec_itc = comp_itc(j,combd, n, noy, est_n, err_n, df, comb,Dsel, D, beta, p)

%SC
term3 = log(df) -(n-1)*log(n-df);
SC1 = n*log(err_n) +df*log( (est_n/df)/(err_n/(n-df)) ) +term3;
diff_n = noy-err_n;
SC2 = n*log(err_n) +df*log( (diff_n/df)/(err_n/(n-df)) ) +term3;

%ESC
zeta1 = 0.5;
% zeta2 = df/j;
ESC1 = SC1 +2*zeta1*comb;
ESC2 = SC2 +2*zeta1*comb;

%gMDL
gMDL1 = (n/2)*log(noy) + (1/2)*log(n);
gMDL2 = gMDL1;
if (est_n/noy) >= (df/n)
    gMDL1 = (n/2)*log(err_n*n/(n-df)) +(df/2)*log( (est_n/df)/(err_n/(n-df)) ) +log(n);
end
if (diff_n/noy) >= (df/n)
    gMDL2 = (n/2)*log(err_n*n/(n-df)) +(df/2)*log( (diff_n/df)/(err_n/(n-df)) ) +log(n);
end

%EgMDL
EgMDL1 = gMDL1 +zeta1*comb;
EgMDL2 = gMDL2 +zeta1*comb;

%BIC     
BIC = n*log(err_n) +df*log(n);

%EBIC1
EBIC1 = BIC +2*zeta1*comb;

%EBIC2
gam = 1; %gam should take values in [0,1]
EBIC2 = n*log(err_n) + df*log(n) +df*4*gam*log(p);

%AICC
AICC = log(err_n) +( 1+df/n )/( 1-(df+2)/n );

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%  NEW CRITERIA %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%KIC
%Seghouane & Bekara 2004, Eq. (4)
KIC = n*log(err_n) + 3*(df+1);


%KICC
%Seghouane & Bekara 2004, Eq. (7)
KICC = n*log(err_n) + (2*(df+1)*n)/(n-df-2) - n*psi( (n-df)/2 );

%MMLu
%Giurcaneanu et al 2011, Page 1678
%Schmidt & Makalic 2009, Eq. (11)
te1 = ((n-df)/2)*log(2*pi);
te2 = ((n-df)/2)*( 1+ log(n*err_n/(n-df)) );
te3 = (df/2)*log(pi*n*noy);
te4 = -gammaln(1+(df/2));
te5 = (1/2)*log(df+1);
MMLU = te1 + te2 + te3 + te4 + te5;

%MMLg
%Giurcaneanu et al 2011, Page 1678
%Schmidt & Makalic 2009, Eq. (20)
MMLG1 = (n/2)*( 1+log((n*err_n)/(n-df+2)) ) + (1/2)*log(n-1) + (1/2); 
MMLG2 = MMLG1;
md = max(df-2,1);
if ( (df>0) &&  ( est_n/md > (err_n/(n-df+2)) ) )
    MMLG1 = ((n-df+2)/2)*( 1+log((n*err_n)/(n-df+2)) ) + ((df-2)/2)*log(n*est_n/md) + (1/2)*log((n-df)*df^2);
end
if ( (df>0) &&  ( diff_n/md > (err_n/(n-df+2)) ) )
    MMLG2 = ((n-df+2)/2)*( 1+log((n*err_n)/(n-df+2)) ) + ((df-2)/2)*log(n*diff_n/md) + (1/2)*log((n-df)*df^2);
end


%Extended criteria (apply gammaln)
ESC1d = SC1 +2*combd;
ESC2d = SC2 +2*combd;
EgMDL1d = gMDL1 +combd;
EgMDL2d = gMDL2 +combd;
EBIC1d = BIC +2*combd;


vec_itc = [ AICC;KIC;KICC; BIC; EBIC1;EBIC1d; EBIC2;SC1; SC2; ESC1; ESC2; ESC1d;ESC2d; gMDL1; gMDL2; EgMDL1; EgMDL2; EgMDL1d;EgMDL2d;MMLU;MMLG1;MMLG2];

end %function


    


