function [Matbeta, vec_err_n, vec_df, ITC, vec_combr, vec_combd] = mpa( y, X, mub, nu )

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Input:
% y         = response vector
% X         = full dictionary
% mub       = max. no. of iterations
% nu        = controls how greedy the algorithm is

%Output:
% Matbeta   = p X mub (estimated coefficients)
% vec_err_n   = mub X 1 (RSS)
% vec_df    = mub X 1 (df)
% ITC       = has mub cols
% Significance of the rows:
% AICC;KIC;KICC;BIC;EBIC1;EBIC1d;EBIC2;SC1;SC2;ESC1;ESC2;ESC1d;ESC2d;gMDL1;
% gMDL2;EgMDL1;EgMDL2;EgMDL1d;EgMDL2d;MMLU;MMLG1;MMLG2;

% vec_combr  = mub X 1 (log of binomial coefficient)
% vec_combd  = mub X 1 (log of binomial coefficient computed with gamma function)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[n,p] = size(X);

vec_k = zeros(mub,1);
Matbeta = zeros(p,mub);
vec_err_n = Inf*ones(mub,1);
vec_df = Inf*ones(mub,1);
vec_combr = zeros(mub,1);
vec_combd = zeros(mub,1);
ITC = Inf*ones(22,mub);

%Transform data
y = y-mean(y);
for jj=1:p
    X(:,jj) = X(:,jj) - mean ( X(:,jj) );
    X(:,jj) = sqrt(n)*( X(:,jj)/norm(X(:,jj)) );
end

c = (X'*y)/n;
D = (X'*X)/n;

noy = norm(y)^2/n;
beta = zeros(p,1);
delta = zeros(p,1);
k=0;
S = zeros(p,1);
sel = [];
df = 0;


if (n >= p)
    w = zeros(p,1);
    G = zeros(p,p);
else
    A = eye(n,n);
end

for j = 1:mub
    if (k>p-1)
        return;
    end
    if df>n-3
        return;
    end
    a = c-delta;
    [~,sj] = max(abs(a));
    beta(sj) = beta(sj) + nu*a(sj);
    delta = delta + nu*a(sj)*D(:,sj);

    est_n = beta'*delta;
    err_n = noy + est_n - 2*beta'*c;
    
    if err_n<=0
        return;
    end
    
    
     Matbeta(:,j) = beta; %save
     vec_err_n(j) = err_n; %save
     
     
     %%Compute df:
     if (n >= p)
        v = -nu*D(sj,:)*G;
        v = v + nu^2*D(sj,:).*w';
        G(sj,:) = G(sj,:) + v;
        w(sj) = w(sj)+1;
        df = df + nu - v*D(:,sj); 
    else
        P = X(:,sj)*X(:,sj)';
        A = A - (nu/n)*P*A;  
        df = n-trace(A);
     end
    vec_df(j) = df; %save
    
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
     if S(sj) == 0
         S(sj) = 1;
         k = sum(S);
     if k==1
            combr = log(p);
        else
            combr = combr + log( (p-k+1)/k );
     end
      
     end
     
     vec_combr(j) = combr;   
     vec_k(j)=k;

     sel = [sel, sj];
     indx = unique(sel);
    
    %Extended criteria (apply gammaln)
    combd = gammaln(p+1) -gammaln(df+1) -gammaln(p-df+1);
    vec_combd(j) = combd;
    
    vec_itc = comp_itc(combd,n, noy, est_n, err_n, df, nu, combr, p, D(indx,indx),D, beta,j);
    ITC(:,j) = vec_itc; %save
    
end %for

end %function



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function vec_itc = comp_itc(combd, n, noy, est_n, err_n, df, nu, comb, p,Dsel,D,bet,j)
%AICC
AICC = log(err_n) +( 1+df/n )/( 1-(df+2)/n );


%KIC
%Seghouane & Bekara 2004, Eq. (4)
KIC = n*log(err_n) + 3*(df+1);

%KICC
%Seghouane & Bekara 2004, Eq. (7)
KICC = n*log(err_n) + (2*(df+1)*n)/(n-df-2) - n*psi( (n-df)/2 );

%BIC
BIC = n*log(err_n) +df*log(n);

%EBIC1
zeta1 = nu;
EBIC1 = BIC +2*zeta1*comb;

%EBIC1d
EBIC1d = BIC +2*combd;

%EBIC2
gam = 1; %gam should take values in [0,1]
EBIC2 = n*log(err_n) +df*log(n) +df*4*gam*log(p);


%SC
term3 = log(df) -(n-1)*log(n-df);
SC1 = n*log(err_n) +df*log( (est_n/df)/(err_n/(n-df)) ) +term3;
diff_n = noy-err_n;
SC2 = n*log(err_n) +df*log( (diff_n/df)/(err_n/(n-df)) ) +term3;

%ESC
ESC1 = SC1 +2*zeta1*comb;
ESC2 = SC2 +2*zeta1*comb;

%ESC1d ESC2d
ESC1d = SC1 +2*combd;
ESC2d = SC2 +2*combd;

%gMDL
gMDL1 = (n/2)*log(noy) + (1/2)*log(n);
gMDL2 = gMDL1;
if (est_n/noy) >= (df/n),
    gMDL1 = (n/2)*log(err_n*n/(n-df)) +(df/2)*log( (est_n/df)/(err_n/(n-df)) ) +log(n);
end
if (diff_n/noy) >= (df/n),
    gMDL2 = (n/2)*log(err_n*n/(n-df)) +(df/2)*log( (diff_n/df)/(err_n/(n-df)) ) +log(n);
end

%EgMDL
EgMDL1 = gMDL1 +zeta1*comb;
EgMDL2 = gMDL2 +zeta1*comb;

%EgMDL1d EgMDL2d
EgMDL1d = gMDL1 +combd;
EgMDL2d = gMDL2 +combd;


%MMLu
%Giurcaneanu et al 2011, Page 1678
%Schmidt & Makalic 2009, Eq. (11)
te1 = ((n-df)/2)*log(2*pi);
te2 = ((n-df)/2)*( 1+ log(n*err_n/(n-df)) );
te3 = (df/2)*log(pi*n*noy);
te4 = -gammaln(1+(df/2));
te5 = (1/2)*log(df+1);
MMLU = te1 + te2 + te3 + te4 + te5;

%MMLg
%Giurcaneanu et al 2011, Page 1678
%Schmidt & Makalic 2009, Eq. (20)
MMLG1 = (n/2)*( 1+log((n*err_n)/(n-df+2)) ) + (1/2)*log(n-1) + (1/2); 
MMLG2 = MMLG1;
md = max(df-2,1);
if ( (df>0) &&  ( est_n/md > (err_n/(n-df+2)) ) )
    MMLG1 = ((n-df+2)/2)*( 1+log((n*err_n)/(n-df+2)) ) + ((df-2)/2)*log(n*est_n/md) + (1/2)*log((n-df)*df^2);
end
if ( (df>0) &&  ( diff_n/md > (err_n/(n-df+2)) ) )
    MMLG2 = ((n-df+2)/2)*( 1+log((n*err_n)/(n-df+2)) ) + ((df-2)/2)*log(n*diff_n/md) + (1/2)*log((n-df)*df^2);
end
    


vec_itc = [AICC;KIC;KICC;BIC;EBIC1;EBIC1d;EBIC2;SC1;SC2;ESC1;ESC2;ESC1d;ESC2d;gMDL1;gMDL2;EgMDL1;EgMDL2;EgMDL1d;EgMDL2d;MMLU;MMLG1;MMLG2];

end %function
    
    